@@ -8,8 +8,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_COMMANDS_CARDCOMMAND_H__
-#define __KLEOPATRA_COMMANDS_CARDCOMMAND_H__
+#pragma once
 
 #include "command.h"
 
@@ -34,4 +33,3 @@ protected:
 
 } // namespace Kleo
 
-#endif /* __KLEOPATRA_COMMANDS_CARDCOMMAND_H__ */
