@@ -29,10 +29,10 @@ protected:
     class Private;
     inline Private *d_func();
     inline const Private *d_func() const;
+
+#ifdef QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
     Q_PRIVATE_SLOT(d_func(), void importResult(GpgME::ImportResult))
-    Q_PRIVATE_SLOT(d_func(), void keyListDone(GpgME::KeyListResult,
-                                              std::vector<GpgME::Key> keys,
-                                              QString, GpgME::Error))
+#endif
 
 protected:
     explicit ImportCertificatesCommand(Private *pp);
