@@ -28,7 +28,7 @@ public:
         return AnyCardHasNullPin;
     }
 
-    QDialog *dialog() const;
+    QDialog *dialog();
 
 private:
     void doStart() override;
@@ -38,10 +38,6 @@ private:
     class Private;
     inline Private *d_func();
     inline const Private *d_func() const;
-    Q_PRIVATE_SLOT(d_func(), void slotDialogRejected())
-    Q_PRIVATE_SLOT(d_func(), void slotDialogAccepted())
-    Q_PRIVATE_SLOT(d_func(), void slotNksPinRequested())
-    Q_PRIVATE_SLOT(d_func(), void slotSigGPinRequested())
 };
 
 }
